/*
Copyright (C) 2006 - 2009 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "DialogASCIIString.h"
#include "DebuggerCoreInterface.h"
#include "MemoryRegions.h"
#include "Debugger.h"
#include "ByteShiftArray.h"
#include "Util.h"
#include "State.h"
#include "Debugger.h"

#include <QVector>
#include <QMessageBox>
#include <cstring>


//------------------------------------------------------------------------------
// Name: DialogASCIIString(QWidget *parent)
// Desc: constructor
//------------------------------------------------------------------------------
DialogASCIIString::DialogASCIIString(QWidget *parent) : QDialog(parent) {
	ui.setupUi(this);
	ui.progressBar->setValue(0);
	ui.listWidget->clear();
}

//------------------------------------------------------------------------------
// Name: doFind()
// Desc: 
//------------------------------------------------------------------------------
void DialogASCIIString::doFind() {

	const QByteArray b = ui.txtASCII->text().toAscii();
	ui.listWidget->clear();

	const int sz = b.size();
	if(sz != 0) {
	
		edb::v1::memoryRegions().sync();

		MemRegion region;
		edb::address_t stack_ptr = edb::v1::currentState().stackPointer();
		
		if(edb::v1::memoryRegions().findRegion(stack_ptr, region)) {
			std::size_t count = (region.end - stack_ptr) / sizeof(edb::address_t);

			try {
				QVector<quint8> chars(sz);
	
				int i = 0;
				while(stack_ptr < region.end) {
					// get the value from the stack
					edb::address_t value;
					if(edb::v1::debuggerBase->readBytes(stack_ptr, &value, sizeof(edb::address_t))) {
						if(edb::v1::debuggerBase->readBytes(value, &chars[0], sz)) {
							if(std::memcmp(&chars[0], b.constData(), sz) == 0) {
								ui.listWidget->addItem(edb::v1::formatPointer(stack_ptr));
							}
						}	
					}
					ui.progressBar->setValue(util::percentage(i++, count));
					stack_ptr += sizeof(edb::address_t);
				}
			} catch(const std::bad_alloc &) {
				QMessageBox::information(0, tr("Memroy Allocation Error"),
					tr("Unable to satisfy memory allocation request for search string."));
			}
		}
	}
}

//------------------------------------------------------------------------------
// Name: on_btnFind_clicked()
// Desc: find button event handler
//------------------------------------------------------------------------------
void DialogASCIIString::on_btnFind_clicked() {

	ui.btnFind->setEnabled(false);
	ui.progressBar->setValue(0);
	doFind();
	ui.progressBar->setValue(100);
	ui.btnFind->setEnabled(true);
}

//------------------------------------------------------------------------------
// Name: on_listWidget_itemDoubleClicked(QListWidgetItem *item)
// Desc: follows the found item in the data view
//------------------------------------------------------------------------------
void DialogASCIIString::on_listWidget_itemDoubleClicked(QListWidgetItem *item) {
	bool ok;
	const edb::address_t addr = edb::v1::stringToAddress(item->text(), ok);
	if(ok) {
		edb::v1::dumpStack(addr, true);
	}
}
