/*
Copyright (C) 2006 - 2009 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "DebuggerCoreBase.h"
#include "X86Breakpoint.h"

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
DebuggerCoreBase::DebuggerCoreBase() : m_ActiveThread(0), m_PID(0) {
}

//------------------------------------------------------------------------------
// Name: ~DebuggerCoreBase()
// Desc: destructor
//------------------------------------------------------------------------------
DebuggerCoreBase::~DebuggerCoreBase() {
}

//------------------------------------------------------------------------------
// Name: clearBreakpoints()
// Desc: 
//------------------------------------------------------------------------------
void DebuggerCoreBase::clearBreakpoints() {
	if(attached()) {	
		m_Breakpoints.clear();
	}
}

//------------------------------------------------------------------------------
// Name: addBreakpoint(edb::address_t address, bool oneTime)
// Desc: 
//------------------------------------------------------------------------------
void DebuggerCoreBase::addBreakpoint(edb::address_t address, bool oneTime) {

	if(!findBreakpoint(address)) {
		QSharedPointer<Breakpoint> bp(new X86Breakpoint(address, oneTime));
		m_Breakpoints[address] = bp;
	}
}

//------------------------------------------------------------------------------
// Name: setBreakpointCondition(edb::address_t address, const QString &condition)
// Desc: 
//------------------------------------------------------------------------------
void DebuggerCoreBase::setBreakpointCondition(edb::address_t address, const QString &condition) {
	QSharedPointer<Breakpoint> bp = findBreakpoint(address);
	if(bp) {
		bp->condition = condition;
	}
}

//------------------------------------------------------------------------------
// Name: setBreakpointCondition(edb::address_t address, const QString &condition)
// Desc: 
//------------------------------------------------------------------------------
QString DebuggerCoreBase::getBreakpointCondition(edb::address_t address) {
	QString ret;
	QSharedPointer<Breakpoint> bp = findBreakpoint(address);
	if(bp) {
		ret = bp->condition;
	}
	
	return ret;
}

//------------------------------------------------------------------------------
// Name: findBreakpoint(edb::address_t address) const
// Desc: 
//------------------------------------------------------------------------------
QSharedPointer<Breakpoint> DebuggerCoreBase::findBreakpoint(edb::address_t address) {
	if(attached()) {
		BreakpointState::iterator it = m_Breakpoints.find(address);
		if(it != m_Breakpoints.end()) {
			return it.value();
		}
	}
	return QSharedPointer<Breakpoint>();
}


//------------------------------------------------------------------------------
// Name: removeBreakpoint(edb::address_t address)
// Desc: 
//------------------------------------------------------------------------------
void DebuggerCoreBase::removeBreakpoint(edb::address_t address) {

	// TODO: assert paused
	if(attached()) {
		BreakpointState::iterator it = m_Breakpoints.find(address);
		if(it != m_Breakpoints.end()) {
			m_Breakpoints.erase(it);
		}
	}
}

//------------------------------------------------------------------------------
// Name: disableBreakpoint(edb::address_t address)
// Desc: 
//------------------------------------------------------------------------------
edb::address_t DebuggerCoreBase::disableBreakpoint(edb::address_t address) {
	if(address != 0) {
		QSharedPointer<Breakpoint> bp = findBreakpoint(address);
		if(bp && bp->disable()) {
			return address;
		}
	}
	return edb::nullptr;
}

//------------------------------------------------------------------------------
// Name: enableBreakpoint(edb::address_t address)
// Desc: 
//------------------------------------------------------------------------------
edb::address_t DebuggerCoreBase::enableBreakpoint(edb::address_t address) {
	if(address != 0) {
		QSharedPointer<Breakpoint> bp = findBreakpoint(address);
		if(bp && bp->enable()) {
			return address;
		}
	}
	return edb::nullptr;
}

//------------------------------------------------------------------------------
// Name: backupBreakpoints() const
// Desc: 
//------------------------------------------------------------------------------
DebuggerCoreBase::BreakpointState DebuggerCoreBase::backupBreakpoints() const {
	return m_Breakpoints;
}

//------------------------------------------------------------------------------
// Name: open(const QString &path, const QString &cwd, const QStringList &args)
// Desc: 
//------------------------------------------------------------------------------
bool DebuggerCoreBase::open(const QString &path, const QString &cwd, const QStringList &args) {
	return open(path, cwd, args, QString());
}

//------------------------------------------------------------------------------
// Name: pid() const
// Desc: returns the pid of the currently debugged process
//------------------------------------------------------------------------------
edb::pid_t DebuggerCoreBase::pid() const {
	return m_PID;
}

//------------------------------------------------------------------------------
// Name: attached() const
// Desc: 
//------------------------------------------------------------------------------
bool DebuggerCoreBase::attached() const {
	return pid() != 0;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
QList<edb::tid_t> DebuggerCoreBase::threadIDs() const {
	return QList<edb::tid_t>();
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
edb::tid_t DebuggerCoreBase::activeThread() const {
	return m_ActiveThread;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void DebuggerCoreBase::setActiveThread(edb::tid_t) {
}

int DebuggerCoreBase::breakpointSize() const {
	return X86Breakpoint::size;
}
