/*
Copyright (C) 2006 - 2009 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "QULongValidator.h"

QULongValidator::QULongValidator(QObject *parent) : QValidator(parent), m_Minimum(0), m_Maximum(0) {
}

QULongValidator::QULongValidator(QULongValidator::value_type minimum, QULongValidator::value_type maximum, QObject *parent) : QValidator(parent), m_Minimum(minimum), m_Maximum(maximum) {
}

QULongValidator::value_type QULongValidator::bottom() const {
	return m_Minimum;
}

void QULongValidator::setBottom(QULongValidator::value_type bottom) {
	m_Minimum = bottom;
}

void QULongValidator::setRange(QULongValidator::value_type bottom, QULongValidator::value_type top) {
	setBottom(bottom);
	setTop(top);
}

void QULongValidator::setTop(QULongValidator::value_type top) {
	m_Maximum = top;
}

QULongValidator::value_type QULongValidator::top() const {
	return m_Maximum;
}

QValidator::State QULongValidator::validate(QString &input, int &pos) const {
	Q_UNUSED(pos);

	if(input.isEmpty()) {
		return QValidator::Acceptable;
	}

	bool ok;
	const value_type temp = input.toULong(&ok);
	if(!ok) {
		return QValidator::Invalid;
	}

	return (temp >= bottom() && temp <= top()) ? QValidator::Acceptable : QValidator::Intermediate;
}
